@@ -1,4 +1,17 @@
+// Libraries
 import _ from 'lodash';
+
+// Utils
+import getFactors from 'app/core/utils/factors';
+import { appendQueryToUrl } from 'app/core/utils/url';
+import kbn from 'app/core/utils/kbn';
+
+// Types
+import { PanelModel } from './PanelModel';
+import { DashboardModel } from './DashboardModel';
+import { DataLink } from '@grafana/data';
+
+// Constants
 import {
   GRID_COLUMN_COUNT,
   GRID_CELL_HEIGHT,
@@ -7,9 +20,7 @@ import {
   MIN_PANEL_HEIGHT,
   DEFAULT_PANEL_SPAN,
 } from 'app/core/constants';
-import { PanelModel } from './PanelModel';
-import { DashboardModel } from './DashboardModel';
-import getFactors from 'app/core/utils/factors';
+import { DataLinkBuiltInVars } from 'app/features/panel/panellinks/link_srv';
 
 export class DashboardMigrator {
   dashboard: DashboardModel;
@@ -18,11 +29,11 @@ export class DashboardMigrator {
     this.dashboard = dashboardModel;
   }
 
-  updateSchema(old) {
+  updateSchema(old: any) {
     let i, j, k, n;
     const oldVersion = this.dashboard.schemaVersion;
     const panelUpgrades = [];
-    this.dashboard.schemaVersion = 18;
+    this.dashboard.schemaVersion = 19;
 
     if (oldVersion === this.dashboard.schemaVersion) {
       return;
@@ -42,7 +53,6 @@ export class DashboardMigrator {
         if (panel.type === 'graphite') {
           panel.type = 'graph';
         }
-
         if (panel.type !== 'graph') {
           return;
         }
@@ -109,7 +119,7 @@ export class DashboardMigrator {
 
     if (oldVersion < 6) {
       // move pulldowns to new schema
-      const annotations = _.find(old.pulldowns, { type: 'annotations' });
+      const annotations: any = _.find(old.pulldowns, { type: 'annotations' });
 
       if (annotations) {
         this.dashboard.annotations = {
@@ -417,6 +427,15 @@ export class DashboardMigrator {
       });
     }
 
+    if (oldVersion < 19) {
+      // migrate change to gauge options
+      panelUpgrades.push(panel => {
+        if (panel.links && _.isArray(panel.links)) {
+          panel.links = panel.links.map(upgradePanelLink);
+        }
+      });
+    }
+
     if (panelUpgrades.length === 0) {
       return;
     }
@@ -612,3 +631,33 @@ class RowArea {
     return place;
   }
 }
+
+function upgradePanelLink(link: any): DataLink {
+  let url = link.url;
+
+  if (!url && link.dashboard) {
+    url = `/dashboard/db/${kbn.slugifyForUrl(link.dashboard)}`;
+  }
+
+  if (!url && link.dashUri) {
+    url = `/dashboard/${link.dashUri}`;
+  }
+
+  if (link.keepTime) {
+    url = appendQueryToUrl(url, `$${DataLinkBuiltInVars.keepTime}`);
+  }
+
+  if (link.includeVars) {
+    url = appendQueryToUrl(url, `$${DataLinkBuiltInVars.includeVars}`);
+  }
+
+  if (link.params) {
+    url = appendQueryToUrl(url, link.params);
+  }
+
+  return {
+    url: url,
+    title: link.title,
+    targetBlank: link.targetBlank,
+  };
+}
