@@ -6,27 +6,33 @@ import { PanelCtrl } from 'app/features/panel/panel_ctrl';
 import { getExploreUrl } from 'app/core/utils/explore';
 import { applyPanelTimeOverrides, getResolution } from 'app/features/dashboard/utils/panel';
 import { ContextSrv } from 'app/core/services/context_srv';
+import { toLegacyResponseData, isDataFrame, TimeRange, LoadingState, DataFrame } from '@grafana/data';
+
+import { LegacyResponseData, DataSourceApi, PanelData, DataQueryResponse } from '@grafana/ui';
+import { Unsubscribable } from 'rxjs';
+import { PanelModel } from 'app/features/dashboard/state';
+import { PanelQueryRunnerFormat } from '../dashboard/state/PanelQueryRunner';
 
 class MetricsPanelCtrl extends PanelCtrl {
   scope: any;
-  datasource: any;
+  datasource: DataSourceApi;
   $q: any;
   $timeout: any;
   contextSrv: ContextSrv;
   datasourceSrv: any;
   timeSrv: any;
   templateSrv: any;
-  range: any;
+  range: TimeRange;
   interval: any;
   intervalMs: any;
   resolution: any;
-  timeInfo: any;
+  timeInfo?: string;
   skipDataOnInit: boolean;
-  dataStream: any;
-  dataSubscription: any;
-  dataList: any;
+  dataList: LegacyResponseData[];
+  querySubscription?: Unsubscribable;
+  dataFormat = PanelQueryRunnerFormat.legacy;
 
-  constructor($scope, $injector) {
+  constructor($scope: any, $injector: any) {
     super($scope, $injector);
 
     this.$q = $injector.get('$q');
@@ -42,9 +48,9 @@ class MetricsPanelCtrl extends PanelCtrl {
   }
 
   private onPanelTearDown() {
-    if (this.dataSubscription) {
-      this.dataSubscription.unsubscribe();
-      this.dataSubscription = null;
+    if (this.querySubscription) {
+      this.querySubscription.unsubscribe();
+      this.querySubscription = null;
     }
   }
 
@@ -70,47 +76,92 @@ class MetricsPanelCtrl extends PanelCtrl {
       });
     }
 
-    // // ignore if we have data stream
-    if (this.dataStream) {
-      return;
-    }
-
     // clear loading/error state
     delete this.error;
     this.loading = true;
 
     // load datasource service
-    this.datasourceSrv
+    return this.datasourceSrv
       .get(this.panel.datasource, this.panel.scopedVars)
       .then(this.updateTimeRange.bind(this))
       .then(this.issueQueries.bind(this))
-      .then(this.handleQueryResult.bind(this))
-      .catch(err => {
-        // if canceled  keep loading set to true
-        if (err.cancelled) {
-          console.log('Panel request cancelled', err);
-          return;
-        }
-
-        this.loading = false;
-        this.error = err.message || 'Request Error';
-        this.inspector = { error: err };
-
-        if (err.data) {
-          if (err.data.message) {
-            this.error = err.data.message;
-          }
-          if (err.data.error) {
-            this.error = err.data.error;
-          }
-        }
-
-        this.events.emit('data-error', err);
-        console.log('Panel data error:', err);
+      .catch((err: any) => {
+        this.processDataError(err);
       });
   }
 
-  updateTimeRange(datasource?) {
+  processDataError(err: any) {
+    // if canceled  keep loading set to true
+    if (err.cancelled) {
+      console.log('Panel request cancelled', err);
+      return;
+    }
+
+    this.loading = false;
+    this.error = err.message || 'Request Error';
+    this.inspector = { error: err };
+
+    if (err.data) {
+      if (err.data.message) {
+        this.error = err.data.message;
+      }
+      if (err.data.error) {
+        this.error = err.data.error;
+      }
+    }
+
+    this.events.emit('data-error', err);
+    console.log('Panel data error:', err);
+  }
+
+  // Updates the response with information from the stream
+  panelDataObserver = {
+    next: (data: PanelData) => {
+      if (data.state === LoadingState.Error) {
+        this.loading = false;
+        this.processDataError(data.error);
+        return;
+      }
+
+      // Ignore data in loading state
+      if (data.state === LoadingState.Loading) {
+        this.loading = true;
+        return;
+      }
+
+      if (data.request) {
+        const { range, timeInfo } = data.request;
+        if (range) {
+          this.range = range;
+        }
+        if (timeInfo) {
+          this.timeInfo = timeInfo;
+        }
+      }
+
+      if (this.dataFormat === PanelQueryRunnerFormat.legacy) {
+        // The result should already be processed, but just in case
+        if (!data.legacy) {
+          data.legacy = data.series.map(v => {
+            if (isDataFrame(v)) {
+              return toLegacyResponseData(v);
+            }
+            return v;
+          });
+        }
+
+        // Make the results look like they came directly from a <6.2 datasource request
+        // NOTE: any object other than 'data' is no longer supported supported
+        this.handleQueryResult({
+          data: data.legacy,
+        });
+      } else {
+        this.handleDataFrame(data.series);
+      }
+    },
+  };
+
+  updateTimeRange(datasource?: DataSourceApi) {
     this.datasource = datasource || this.datasource;
     this.range = this.timeSrv.timeRange();
     this.resolution = getResolution(this.panel);
@@ -139,45 +190,40 @@ class MetricsPanelCtrl extends PanelCtrl {
     this.intervalMs = res.intervalMs;
   }
 
-  issueQueries(datasource) {
+  issueQueries(datasource: DataSourceApi) {
     this.datasource = datasource;
 
-    if (!this.panel.targets || this.panel.targets.length === 0) {
-      return this.$q.when([]);
+    const panel = this.panel as PanelModel;
+    const queryRunner = panel.getQueryRunner();
+
+    if (!this.querySubscription) {
+      this.querySubscription = queryRunner.subscribe(this.panelDataObserver, this.dataFormat);
     }
 
-    // make shallow copy of scoped vars,
-    // and add built in variables interval and interval_ms
-    const scopedVars = Object.assign({}, this.panel.scopedVars, {
-      __interval: { text: this.interval, value: this.interval },
-      __interval_ms: { text: this.intervalMs, value: this.intervalMs },
-    });
-
-    const metricsQuery = {
-      timezone: this.dashboard.getTimezone(),
-      panelId: this.panel.id,
+    return queryRunner.run({
+      datasource: panel.datasource,
+      queries: panel.targets,
+      panelId: panel.id,
       dashboardId: this.dashboard.id,
-      range: this.range,
-      rangeRaw: this.range.raw,
-      interval: this.interval,
-      intervalMs: this.intervalMs,
-      targets: this.panel.targets,
-      maxDataPoints: this.resolution,
-      scopedVars: scopedVars,
-      cacheTimeout: this.panel.cacheTimeout,
-    };
-
-    return datasource.query(metricsQuery);
+      timezone: this.dashboard.timezone,
+      timeRange: this.range,
+      widthPixels: this.resolution, // The pixel width
+      maxDataPoints: panel.maxDataPoints,
+      minInterval: panel.interval,
+      scopedVars: panel.scopedVars,
+      cacheTimeout: panel.cacheTimeout,
+    });
   }
 
-  handleQueryResult(result) {
-    this.loading = false;
-
-    // check for if data source returns subject
-    if (result && result.subscribe) {
-      this.handleDataStream(result);
-      return;
+  handleDataFrame(data: DataFrame[]) {
+    if (this.dashboard && this.dashboard.snapshot) {
+      this.panel.snapshotData = data;
     }
+    // Subclasses that asked for DataFrame will override
+  }
+
+  handleQueryResult(result: DataQueryResponse) {
+    this.loading = false;
 
     if (this.dashboard.snapshot) {
       this.panel.snapshotData = result.data;
@@ -191,33 +237,6 @@ class MetricsPanelCtrl extends PanelCtrl {
     this.events.emit('data-received', result.data);
   }
 
-  handleDataStream(stream) {
-    // if we already have a connection
-    if (this.dataStream) {
-      console.log('two stream observables!');
-      return;
-    }
-
-    this.dataStream = stream;
-    this.dataSubscription = stream.subscribe({
-      next: data => {
-        console.log('dataSubject next!');
-        if (data.range) {
-          this.range = data.range;
-        }
-        this.events.emit('data-received', data.data);
-      },
-      error: error => {
-        this.events.emit('data-error', error);
-        console.log('panel: observer got error');
-      },
-      complete: () => {
-        console.log('panel: observer got complete');
-        this.dataStream = null;
-      },
-    });
-  }
-
   getAdditionalMenuItems() {
     const items = [];
     if (this.contextSrv.hasAccessToExplore() && this.datasource) {
