@@ -5,10 +5,9 @@ import { connect } from 'react-redux';
 import find from 'lodash/find';
 
 // Types
-import { StoreState, UrlQueryMap } from 'app/types';
+import { UrlQueryMap } from '@grafana/runtime';
+import { StoreState } from 'app/types';
 import {
-  NavModel,
-  NavModelItem,
   PluginType,
   GrafanaPlugin,
   PluginInclude,
@@ -19,6 +18,7 @@ import {
   AppPlugin,
   PluginIncludeType,
 } from '@grafana/ui';
+import { NavModel, NavModelItem } from '@grafana/data';
 
 import Page from 'app/core/components/Page/Page';
 import { getPluginSettings } from './PluginSettingsCache';
@@ -29,6 +29,7 @@ import { AppConfigCtrlWrapper } from './wrappers/AppConfigWrapper';
 import { PluginDashboards } from './PluginDashboards';
 import { appEvents } from 'app/core/core';
 import { config } from 'app/core/config';
+import { ContextSrv } from '../../core/services/context_srv';
 
 export function getLoadingNav(): NavModel {
   const node = {
@@ -69,18 +70,19 @@ interface Props {
   pluginId: string;
   query: UrlQueryMap;
   path: string; // the URL path
+  $contextSrv: ContextSrv;
 }
 
 interface State {
   loading: boolean;
   plugin?: GrafanaPlugin;
   nav: NavModel;
-  defaultTab: string; // The first configured one or readme
+  defaultPage: string; // The first configured one or readme
 }
 
-const TAB_ID_README = 'readme';
-const TAB_ID_DASHBOARDS = 'dashboards';
-const TAB_ID_CONFIG_CTRL = 'config';
+const PAGE_ID_README = 'readme';
+const PAGE_ID_DASHBOARDS = 'dashboards';
+const PAGE_ID_CONFIG_CTRL = 'config';
 
 class PluginPage extends PureComponent<Props, State> {
   constructor(props: Props) {
@@ -88,12 +90,12 @@ class PluginPage extends PureComponent<Props, State> {
     this.state = {
       loading: true,
       nav: getLoadingNav(),
-      defaultTab: TAB_ID_README,
+      defaultPage: PAGE_ID_README,
     };
   }
 
   async componentDidMount() {
-    const { pluginId, path, query } = this.props;
+    const { pluginId, path, query, $contextSrv } = this.props;
     const { appSubUrl } = config;
 
     const plugin = await loadPlugin(pluginId);
@@ -104,105 +106,25 @@ class PluginPage extends PureComponent<Props, State> {
       });
       return; // 404
     }
-    const { meta } = plugin;
 
-    let defaultTab: string;
-    const tabs: NavModelItem[] = [];
-    if (true) {
-      tabs.push({
-        text: 'Readme',
-        icon: 'fa fa-fw fa-file-text-o',
-        url: `${appSubUrl}${path}?tab=${TAB_ID_README}`,
-        id: TAB_ID_README,
-      });
-    }
-
-    // Only show Config/Pages for app
-    if (meta.type === PluginType.app) {
-      // Legacy App Config
-      if (plugin.angularConfigCtrl) {
-        tabs.push({
-          text: 'Config',
-          icon: 'gicon gicon-cog',
-          url: `${appSubUrl}${path}?tab=${TAB_ID_CONFIG_CTRL}`,
-          id: TAB_ID_CONFIG_CTRL,
-        });
-        defaultTab = TAB_ID_CONFIG_CTRL;
-      }
-
-      if (plugin.configTabs) {
-        for (const tab of plugin.configTabs) {
-          tabs.push({
-            text: tab.title,
-            icon: tab.icon,
-            url: path + '?tab=' + tab.id,
-            id: tab.id,
-          });
-          if (!defaultTab) {
-            defaultTab = tab.id;
-          }
-        }
-      }
-
-      // Check for the dashboard tabs
-      if (find(meta.includes, { type: 'dashboard' })) {
-        tabs.push({
-          text: 'Dashboards',
-          icon: 'gicon gicon-dashboard',
-          url: `${appSubUrl}${path}?tab=${TAB_ID_DASHBOARDS}`,
-          id: TAB_ID_DASHBOARDS,
-        });
-      }
-    }
-
-    if (!defaultTab) {
-      defaultTab = tabs[0].id; // the first tab
-    }
-
-    const node = {
-      text: meta.name,
-      img: meta.info.logos.large,
-      subTitle: meta.info.author.name,
-      breadcrumbs: [{ title: 'Plugins', url: '/plugins' }],
-      url: `${appSubUrl}${path}`,
-      children: this.setActiveTab(query.tab as string, tabs, defaultTab),
-    };
+    const { defaultPage, nav } = getPluginTabsNav(plugin, appSubUrl, path, query, $contextSrv.hasRole('Admin'));
 
     this.setState({
       loading: false,
       plugin,
-      defaultTab,
-      nav: {
-        node: node,
-        main: node,
-      },
+      defaultPage,
+      nav,
     });
   }
 
-  setActiveTab(tabId: string, tabs: NavModelItem[], defaultTabId: string): NavModelItem[] {
-    let found = false;
-    const selected = tabId || defaultTabId;
-    const changed = tabs.map(tab => {
-      const active = !found && selected === tab.id;
-      if (active) {
-        found = true;
-      }
-      return { ...tab, active };
-    });
-    if (!found) {
-      changed[0].active = true;
-    }
-    return changed;
-  }
-
   componentDidUpdate(prevProps: Props) {
-    const prevTab = prevProps.query.tab as string;
-    const tab = this.props.query.tab as string;
-    if (prevTab !== tab) {
-      const { nav, defaultTab } = this.state;
+    const prevPage = prevProps.query.page as string;
+    const page = this.props.query.page as string;
+    if (prevPage !== page) {
+      const { nav, defaultPage } = this.state;
       const node = {
         ...nav.node,
-        children: this.setActiveTab(tab, nav.node.children, defaultTab),
+        children: setActivePage(page, nav.node.children, defaultPage),
       };
       this.setState({
         nav: {
@@ -224,21 +146,21 @@ class PluginPage extends PureComponent<Props, State> {
     const active = nav.main.children.find(tab => tab.active);
     if (active) {
       // Find the current config tab
-      if (plugin.configTabs) {
-        for (const tab of plugin.configTabs) {
+      if (plugin.configPages) {
+        for (const tab of plugin.configPages) {
           if (tab.id === active.id) {
-            return <tab.body meta={plugin.meta} query={query} />;
+            return <tab.body plugin={plugin} query={query} />;
           }
         }
       }
 
       // Apps have some special behavior
       if (plugin.meta.type === PluginType.app) {
-        if (active.id === TAB_ID_DASHBOARDS) {
+        if (active.id === PAGE_ID_DASHBOARDS) {
           return <PluginDashboards plugin={plugin.meta} />;
         }
 
-        if (active.id === TAB_ID_CONFIG_CTRL && plugin.angularConfigCtrl) {
+        if (active.id === PAGE_ID_CONFIG_CTRL && plugin.angularConfigCtrl) {
           return <AppConfigCtrlWrapper app={plugin as AppPlugin} />;
         }
       }
@@ -368,6 +290,8 @@ class PluginPage extends PureComponent<Props, State> {
 
   render() {
     const { loading, nav, plugin } = this.state;
+    const { $contextSrv } = this.props;
+    const isAdmin = $contextSrv.hasRole('Admin');
     return (
       <Page navModel={nav}>
         <Page.Contents isLoading={loading}>
@@ -378,7 +302,7 @@ class PluginPage extends PureComponent<Props, State> {
                 {plugin && (
                   <section className="page-sidebar-section">
                     {this.renderVersionInfo(plugin.meta)}
-                    {this.renderSidebarIncludes(plugin.meta.includes)}
+                    {isAdmin && this.renderSidebarIncludes(plugin.meta.includes)}
                     {this.renderSidebarDependencies(plugin.meta.dependencies)}
                     {this.renderSidebarLinks(plugin.meta.info)}
                   </section>
@@ -392,6 +316,106 @@ class PluginPage extends PureComponent<Props, State> {
   }
 }
 
+function getPluginTabsNav(
+  plugin: GrafanaPlugin,
+  appSubUrl: string,
+  path: string,
+  query: UrlQueryMap,
+  isAdmin: boolean
+): { defaultPage: string; nav: NavModel } {
+  const { meta } = plugin;
+  let defaultPage: string;
+  const pages: NavModelItem[] = [];
+
+  if (true) {
+    pages.push({
+      text: 'Readme',
+      icon: 'fa fa-fw fa-file-text-o',
+      url: `${appSubUrl}${path}?page=${PAGE_ID_README}`,
+      id: PAGE_ID_README,
+    });
+  }
+
+  // We allow non admins to see plugins but only their readme. Config is hidden even though the API needs to be
+  // public for plugins to work properly.
+  if (isAdmin) {
+    // Only show Config/Pages for app
+    if (meta.type === PluginType.app) {
+      // Legacy App Config
+      if (plugin.angularConfigCtrl) {
+        pages.push({
+          text: 'Config',
+          icon: 'gicon gicon-cog',
+          url: `${appSubUrl}${path}?page=${PAGE_ID_CONFIG_CTRL}`,
+          id: PAGE_ID_CONFIG_CTRL,
+        });
+        defaultPage = PAGE_ID_CONFIG_CTRL;
+      }
+
+      if (plugin.configPages) {
+        for (const page of plugin.configPages) {
+          pages.push({
+            text: page.title,
+            icon: page.icon,
+            url: path + '?page=' + page.id,
+            id: page.id,
+          });
+          if (!defaultPage) {
+            defaultPage = page.id;
+          }
+        }
+      }
+
+      // Check for the dashboard pages
+      if (find(meta.includes, { type: 'dashboard' })) {
+        pages.push({
+          text: 'Dashboards',
+          icon: 'gicon gicon-dashboard',
+          url: `${appSubUrl}${path}?page=${PAGE_ID_DASHBOARDS}`,
+          id: PAGE_ID_DASHBOARDS,
+        });
+      }
+    }
+  }
+
+  if (!defaultPage) {
+    defaultPage = pages[0].id; // the first tab
+  }
+
+  const node = {
+    text: meta.name,
+    img: meta.info.logos.large,
+    subTitle: meta.info.author.name,
+    breadcrumbs: [{ title: 'Plugins', url: '/plugins' }],
+    url: `${appSubUrl}${path}`,
+    children: setActivePage(query.page as string, pages, defaultPage),
+  };
+
+  return {
+    defaultPage,
+    nav: {
+      node: node,
+      main: node,
+    },
+  };
+}
+
+function setActivePage(pageId: string, pages: NavModelItem[], defaultPageId: string): NavModelItem[] {
+  let found = false;
+  const selected = pageId || defaultPageId;
+  const changed = pages.map(p => {
+    const active = !found && selected === p.id;
+    if (active) {
+      found = true;
+    }
+    return { ...p, active };
+  });
+  if (!found) {
+    changed[0].active = true;
+  }
+  return changed;
+}
+
 function getPluginIcon(type: string) {
   switch (type) {
     case 'datasource':
