@@ -1,7 +1,8 @@
 import kbn from 'app/core/utils/kbn';
 import _ from 'lodash';
 import { variableRegex } from 'app/features/templating/variable';
-import { TimeRange, ScopedVars } from '@grafana/ui/src';
+import { ScopedVars } from '@grafana/ui';
+import { TimeRange } from '@grafana/data';
 
 function luceneEscape(value) {
   return value.replace(/([\!\*\+\-\=<>\s\&\|\(\)\[\]\{\}\^\~\?\:\\/"])/g, '\\$1');
@@ -13,7 +14,7 @@ export class TemplateSrv {
   private regex = variableRegex;
   private index = {};
   private grafanaVariables = {};
-  private builtIns = {};
+  private builtIns: any = {};
   private timeRange: TimeRange = null;
 
   constructor() {
@@ -28,6 +29,10 @@ export class TemplateSrv {
     this.updateIndex();
   }
 
+  getBuiltInIntervalValue() {
+    return this.builtIns.__interval.value;
+  }
+
   updateIndex() {
     const existsOrEmpty = value => value || value === '';
 
@@ -309,7 +314,7 @@ export class TemplateSrv {
   }
 
   distributeVariable(value, variable) {
-    value = _.map(value, (val, index) => {
+    value = _.map(value, (val: any, index: number) => {
       if (index !== 0) {
         return variable + '=' + val;
       } else {
